@@ -33,6 +33,7 @@ module System.FSNotify
        -- * Events
          Event(..)
        , EventChannel
+       , eventIsDirectory
        , eventTime
        , eventPath
        , Action
@@ -60,15 +61,14 @@ module System.FSNotify
 
 import Prelude hiding (FilePath)
 
-import Data.Maybe
 import Control.Concurrent
 import Control.Concurrent.Async
 import Control.Exception
-import Control.Applicative
 import Control.Monad
-import System.FilePath
+import Data.Maybe
 import System.FSNotify.Polling
 import System.FSNotify.Types
+import System.FilePath
 
 import System.FSNotify.Listener (StopListening)
 
@@ -172,15 +172,13 @@ watchTreeChan (WatchManager db wm _) = listenRecursive db wm
 -- | Watch the immediate contents of a directory by committing an Action for each event.
 -- Watching the immediate contents of a directory will only report events
 -- associated with files within the specified directory, and not files
--- within its subdirectories. No two events pertaining to the same FilePath will
--- be executed concurrently.
+-- within its subdirectories.
 watchDir :: WatchManager -> FilePath -> ActionPredicate -> Action -> IO StopListening
 watchDir wm = threadChan listen wm
 
 -- | Watch all the contents of a directory by committing an Action for each event.
 -- Watching all the contents of a directory will report events associated with
--- files within the specified directory and its subdirectories. No two events
--- pertaining to the same FilePath will be executed concurrently.
+-- files within the specified directory and its subdirectories.
 watchTree :: WatchManager -> FilePath -> ActionPredicate -> Action -> IO StopListening
 watchTree wm = threadChan listenRecursive wm
 
@@ -190,8 +188,7 @@ threadChan
       -- (^ this is the type of listen and listenRecursive)
   ->  WatchManager -> FilePath -> ActionPredicate -> Action -> IO StopListening
 threadChan listenFn (WatchManager db listener cleanupVar) path actPred action =
-  modifyMVar cleanupVar $ \mbCleanup ->
-  case mbCleanup of
+  modifyMVar cleanupVar $ \mbCleanup -> case mbCleanup of
     -- check if we've been stopped
     Nothing -> return (Nothing, return ()) -- or throw an exception?
     Just cleanup -> do
