@@ -1,19 +1,20 @@
 {-# LANGUAGE OverloadedStrings, ImplicitParams #-}
 module EventUtils where
 
-import Prelude hiding (FilePath)
-import Test.Tasty.HUnit
+import Control.Applicative
 import Control.Concurrent
 import Control.Concurrent.Async hiding (poll)
-import Control.Applicative
 import Control.Monad
 import Data.IORef
 import Data.List (sortBy)
+import Data.Monoid
 import Data.Ord (comparing)
-import System.FilePath
-import System.FSNotify
-import System.IO.Unsafe
+import Prelude hiding (FilePath)
 import System.Directory
+import System.FSNotify
+import System.FilePath
+import System.IO.Unsafe
+import Test.Tasty.HUnit
 import Text.Printf
 
 delay :: (?timeInterval :: Int) => IO ()
@@ -26,23 +27,32 @@ data EventPattern = EventPattern
   , patPredicate :: Event -> Bool
   }
 
-evAdded, evRemoved, evModified :: FilePath -> EventPattern
-evAdded path =
-  EventPattern
-    path
-    "Added"
-    (\x -> case x of Added path' _ -> path == path'; _ -> False)
-evRemoved path =
-  EventPattern
-    path
-    "Removed"
-    (\x -> case x of Removed path' _ -> path == path'; _ -> False)
-evModified path =
-  EventPattern
-    path
-    "Modified"
-    (\x -> case x of Modified path' _ -> path == path'; _ -> False)
+evAdded, evRemoved, evModified, evAddedOrModified :: Bool -> FilePath -> EventPattern
+evAdded isDirectory path = EventPattern path "Added"
+  (\x -> case x of
+      Added path' _ isDir | isDirectory == isDir -> pathMatches isDirectory path path'
+      _ -> False
+  )
+evRemoved isDirectory path = EventPattern path "Removed"
+  (\x -> case x of
+      Removed path' _ isDir | isDirectory == isDir -> pathMatches isDirectory path path'
+      _ -> False
+  )
+evModified isDirectory path = EventPattern path "Modified"
+  (\x ->
+     case x of
+       Modified path' _ isDir | isDirectory == isDir -> pathMatches isDirectory path path'
+       _ -> False
+  )
+evAddedOrModified isDirectory path = EventPattern path "AddedOrModified"
+  (\x -> case x of
+      Added path' _ isDir | isDirectory == isDir -> pathMatches isDirectory path path'
+      Modified path' _ isDir | isDirectory == isDir -> pathMatches isDirectory path path'
+      _ -> False
+  )
 
+pathMatches True path path' = path == path' || (path <> [pathSeparator]) == path'
+pathMatches False path path' = path == path'
 
 matchEvents :: [EventPattern] -> [Event] -> Assertion
 matchEvents expected actual = do
@@ -53,7 +63,7 @@ matchEvents expected actual = do
       (show actual)
   sequence_ $ (\f -> zipWith f expected actual) $ \pat ev ->
     assertBool
-      (printf "Unexpected event.\n  Expected :%s\n  Actual: %s\n"
+      (printf "Unexpected event.\n  Expected: %s\n  Actual: %s\n"
         (show expected)
         (show actual))
       (patPredicate pat ev)
@@ -69,11 +79,9 @@ gatherEvents
   -> FilePath
   -> IO (Async [Event])
 gatherEvents poll watch path = do
-  mgr <- startManagerConf defaultConfig
-    { confDebounce = NoDebounce
-    , confUsePolling = poll
-    , confPollInterval = 2 * 10^(5 :: Int)
-    }
+  mgr <- startManagerConf defaultConfig { confDebounce = NoDebounce
+                                        , confUsePolling = poll
+                                        , confPollInterval = 2 * 10^(5 :: Int) }
   eventsVar <- newIORef []
   stop <- watch mgr path (const True) (\ev -> atomicModifyIORef eventsVar (\evs -> (ev:evs, ())))
   async $ do
